import tornado.web


class HandlerFactory(object):
    """
    HandleFactory handles aggregating transmute functions
    into a set of handlers.
    """

    def __init__(self):
        self._handlers = {}

    def add_handler(self, path, transmute_func):
        if path not in self._handlers:
            self._handlers[path] = _create_handler()

        handler = self._handlers[path]

        _add_transmute_func_to_handler(transmute_func, handler)

    def get_handler_tuples(self):
        handlers = []
        for path, handler in self._handlers.items():
            handlers.append((path, handler))
        return handlers


def _create_handler():

    class Handler(tornado.web.RequestHandler):
        pass

    return Handler

METHOD_MAP = {
    "creates": "PUT",
    "deletes": "delete",
    "updates": "POST",
}


def _add_transmute_func_to_handler(transmute_func, handler):
    handler_method = _generate_handler_method(transmute_func)

    is_get = True
    for attr, method in METHOD_MAP.items():
        if getattr(transmute_func, attr, False):
            is_get = False
            setattr(handler, attr.lower(), handler_method)

    if is_get:
        setattr(handler, "get", handler_method)


def _generate_handler_method(transmute_func):

    def method(self):
        self.write(transmute_func.description)

    return method



import tornado.web
from ..routeset import RouteSet
from .handler_factory import HandlerFactory


class TornadoRouteSet(RouteSet):

    def __init__(self, *args, **kwargs):
        super(TornadoRouteSet, self).__init__(*args, **kwargs)

    def generate_handlers(self):
        handler_factory = HandlerFactory()
        for route_config in self._routes:
            handler_factory.add_handler(
                route_config.path, route_config.transmute_func
            )

        return handler_factory.get_handler_tuples()


def _create_handler(transmute_func):

    class Handler(tornado.web.RequestHandler):

        def get(self):
            self.write("Hello, " + transmute_func.description)

    return Handler



import functools
import json
from ..exceptions import ApiException


def wrap_method(transmute_function):
    """
    handles the conversion of a standard funtion into a tornado
    request hanlder method.
    """
    tf = transmute_function
    is_post_method = tf.creates or tf.updates

    api_exceptions = tuple(list(tf.error_exceptions or []) + [ApiException])

    @functools.wraps(tf.raw_func)
    def wrapper_func(self):

        try:
            pass
        except Exception as e:
            if api_exceptions and isinstance(e, api_exceptions):
                self.write(json.dumps({
                    "success": True,
                    "detail": str(e)
                }))
                self.finish()
            else:
                raise


def _retrieve_request_params(args_not_empty):
    pass
